# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

import sys
import datetime
import serpent
import sys
def usage():
    # sys.argv[0] - usage python file name
    # sys.argv[1] - xclient/xcffi/xpyro
    # sys.argv[2] - host/board
    # sys.argv[3] - ip_address
    # sys.argv[4] - port
    print(f"Usage: The application work in three modes xclient, xcffi and xpyro\n"
          f"python3 {sys.argv[0]} xclient host/board ip_address port => Runs at host or board with xclient\n"
          f"sudo python3 {sys.argv[0]} xcffi => Runs at board with xcffi\n"
          f"python3 {sys.argv[0]} xpyro host/board ip_address port => Runs at host or board with xpyro\n"
          f"Example: python3 {sys.argv[0]} xclient host 169.254.10.2 9090")

# 'xcffi' option will run only in the board
if (len(sys.argv) ==  2):
    if (sys.argv[1] != 'xcffi'):
        usage()
        sys.exit()

elif (len(sys.argv) !=  5):
    usage()
    sys.exit()

# The 'xclient' option will run both in host and board
if (sys.argv[1] == 'xclient'):
    # For 'xclient' option the path of the xclient is required
    # The relative path in the host is given for host case
    if (sys.argv[2] == 'host'):
        sys.path.append('../../../xclient/data_stream/data_transfer_no_dma/')
    # The location of xclient in the filesystem is given for the board case
    elif (sys.argv[2] == 'board'):
        sys.path.append('/usr/share/raft/xclient/data_stream/data_transfer_no_dma/')
    else:
        usage()
        sys.exit()
    import axi_memmap_client
    handle = axi_memmap_client.axi_memmap
    ip_address = sys.argv[3]
    port = sys.argv[4]

elif (sys.argv[1] == 'xcffi'):
    # For 'xcffi' option the path of the cffi python code is required
    sys.path.append('/usr/share/raft/xserver/xcffi/drv_api/data_stream/data_transfer_no_dma/')
    from axi_memmap_c import AXI_MEMMAP_C
    handle = AXI_MEMMAP_C()

# The 'xpyro' option will run both in host and board
elif (sys.argv[1] == 'xpyro'):
    ip_address = sys.argv[3]
    port = sys.argv[4]
    import Pyro4
    # Prepare the uri needed for pyro communication
    uri = f"PYRO:AXI_MEMMAP@{ip_address}:{port}"
    handle = Pyro4.Proxy(uri)
else:
    usage()
    sys.exit()

if (sys.argv[1] == 'xclient'):
    #SetIpAndPort
    handle.SetIpAndPort(ip_address, port)

def convert_to_list(data):
    k = []
    for i in range(int(len(data)/4)):
        k.append(int.from_bytes(data[(i*4):(((i+1)*4))], byteorder='little'))
    return k

def convert_to_bytearray(data):
    b = bytearray()
    for i in range(len(data)):
        c = data[i].to_bytes(4, byteorder='little', signed=False)
        b.extend(c)
    return b

def test_axi_write_read():
    buf = [0xABCDEF12, 0x34567890, 0x567890AB, 0x7890ABCD, 0x342ABC12]
    address = 0xa7c0000
    num_words = 5
    network_order = 0
    if (sys.argv[1] == 'xpyro'):
        buf = convert_to_bytearray(buf)
    ret = handle.axi_write_words(address, num_words, buf, network_order)
    assert ret == 0
    ret, buf = handle.axi_read_words(address, num_words, network_order)
    #This conversion is done in xpyro client
    if (sys.argv[1] == 'xpyro'):
        buf = serpent.tobytes(buf)
        buf = convert_to_list(buf)
    assert ret == 0
    assert buf[0] == 0xABCDEF12
    assert buf[1] == 0x34567890
    assert buf[2] == 0x567890AB
    assert buf[3] == 0x7890ABCD
    assert buf[4] == 0x342ABC12

def test_axi_write_read_bulk():
    buf = []
    num_words_write = 1024*2
    num_words = num_words_write
    for i in range (num_words):
        buf.append(0xFFFFFFFF - i)
    address = 0xa7c0000
    network_order = 0
    print("num_words at client test = ", num_words)
    a = datetime.datetime.now()
    if (sys.argv[1] == 'xpyro'):
        buf = convert_to_bytearray(buf)
    ret = handle.axi_write_words(address, num_words, buf, network_order)
    b = datetime.datetime.now()
    c = b-a
    print("Time taken for writing ", num_words * 4, "bytes = ", c.total_seconds(), " seconds")
    print("Write Throughput = ", round((num_words*4)/c.total_seconds()/1000000, 4), "Mega bytes per second")
    assert ret == 0
    num_words = 1024*1000
    a = datetime.datetime.now()
    ret, buf = handle.axi_read_words(address, num_words, network_order)
    #This conversion is done in xpyro client
    if (sys.argv[1] == 'xpyro'):
        buf = serpent.tobytes(buf)
        buf = convert_to_list(buf)
    b = datetime.datetime.now()
    c = b-a
    print("Time taken for reading ", num_words * 4, "bytes = ", c.total_seconds(), " seconds")
    print("Read Throughput = ", round((num_words*4)/c.total_seconds()/1000000,4), "Mega bytes per second")
    for i in range (num_words_write):
        assert buf[i]  == (0xFFFFFFFF - i)

test_axi_write_read()
test_axi_write_read_bulk()



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

import sys
RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
import logging
import json
from utils import ffi, open_c_library, cdata_to_py, xhelper_handle
from utils import get_python_log_levels

rfclk_handle = open_c_library(RAFT_DIR + "xserver/xcffi/drv_header/rfdc/xrfclk_h_python.h", "/usr/lib/librfclk.so.1")


class RFCLK(object):
    MAX_DATA_SIZE_LMK = 128
    MAX_DATA_SIZE_LMX = 116
    RFCLK_LMX2594_1 = 0
    RFCLK_LMX2594_2 = 1
    RFCLK_LMK = 2
    max_size = 0
    logger = None

    def __init__(self):
        self.logger = self.GetLogger()
        ret = xhelper_handle.XHelper_MetalInit(xhelper_handle.METAL_LOG_ERROR)
        if 0 != ret:
            self.logger.error("RFCLK: XHelper_MetalInit failed. ret = ", ret)
        self.logger.info(f"Inside RFCLK Constructor")
        pass

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            logger.handler_set = True
            logger.disabled = False
        return logger

    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        return get_python_log_levels()

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"PythonLogLevel = {PythonLogLevel}")
        LogLevelsDict = get_python_log_levels()
        if PythonLogLevel == LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def SetMetalLogLevel(self, MetalLogLevel):
        """
        Set the metal log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"SetMetalLogLevel({MetalLogLevel})")
        metal_log_level = ffi.typeof("enum metal_log_level").relements
        if MetalLogLevel == metal_log_level["METAL_LOG_DEBUG"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_DEBUG)
        elif MetalLogLevel == metal_log_level["METAL_LOG_INFO"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_INFO)
        elif MetalLogLevel == metal_log_level["METAL_LOG_NOTICE"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_NOTICE)
        elif MetalLogLevel == metal_log_level["METAL_LOG_WARNING"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_WARNING)
        elif MetalLogLevel == metal_log_level["METAL_LOG_ERROR"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_ERROR)
        elif MetalLogLevel == metal_log_level["METAL_LOG_CRITICAL"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_CRITICAL)
        elif MetalLogLevel == metal_log_level["METAL_LOG_ALERT"]:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_ALERT)
        else:
            xhelper_handle.XHelper_MetalSetLogLevel(xhelper_handle.METAL_LOG_EMERGENCY)
        return

    def GetRfclkMacro(self):
        """
        Return Dictionary with all RFCLK macros in the rfclk header file

        :param : None
        :return: Dictionary with all RFCLK macros in the rfclk header file
        """
        rfclk_macro = {}
        rfclk_macro["XST_SUCCESS"] = rfclk_handle.XST_SUCCESS
        rfclk_macro["XST_FAILURE"] = rfclk_handle.XST_FAILURE
        rfclk_macro["RFCLK_LMX2594_1"] = rfclk_handle.RFCLK_LMX2594_1
        rfclk_macro["RFCLK_LMX2594_2"] = rfclk_handle.RFCLK_LMX2594_2
        rfclk_macro["RFCLK_LMK"] = rfclk_handle.RFCLK_LMK
        rfclk_macro["RFCLK_CHIP_NUM"] = rfclk_handle.RFCLK_CHIP_NUM
        rfclk_macro["LMK_COUNT"] = rfclk_handle.LMK_COUNT
        rfclk_macro["LMK_FREQ_NUM"] = rfclk_handle.LMK_FREQ_NUM
        rfclk_macro["LMX_ADC_NUM"] = rfclk_handle.LMX_ADC_NUM
        rfclk_macro["LMX_DAC_NUM"] = rfclk_handle.LMX_DAC_NUM
        rfclk_macro["LMX2594_COUNT"] = rfclk_handle.LMX2594_COUNT
        rfclk_macro["FREQ_LIST_STR_SIZE"] = rfclk_handle.FREQ_LIST_STR_SIZE
        return rfclk_macro

    def GetEnum_metal_log_level(self):
        """
        Return Dictionary equivalent of enum metal_log_level

        :param : None
        :return: Dictionary equivalent of enum metal_log_level
        """
        metal_log_level = ffi.typeof("enum metal_log_level").relements
        self.logger.debug(f"metal_log_level = {json.dumps(metal_log_level, indent=2)}")
        return metal_log_level

    def XRFClk_Init(self, GpioId):
        ret = rfclk_handle.XRFClk_Init(GpioId)
        self.logger.debug(f"\nret = XRFClk_Init({GpioId})")
        self.logger.debug(f"The return value is: {ret}")
        return ret

    def XRFClk_ResetChip(self, ChipId):
        self.logger.debug(f"\nret = XRFClk_ResetChip({ChipId})")
        ret = rfclk_handle.XRFClk_ResetChip(ChipId)
        self.logger.debug(f"The return value is: {ret}")
        return ret

    def XRFClk_SetConfigOnOneChipFromConfigId(self, ChipId, ConfigId):
        self.logger.debug(
            f"\nret = XRFClk_SetConfigOnOneChipFromConfigId({ChipId}, {ConfigId})"
        )
        ret = rfclk_handle.XRFClk_SetConfigOnOneChipFromConfigId(ChipId, ConfigId)
        self.logger.debug(f"The return value is: {ret}")
        return ret

    def XRFClk_GetConfigFromOneChip(self, ChipId):
        if ChipId == self.RFCLK_LMK:
            self.max_size = self.MAX_DATA_SIZE_LMK
        else:
            self.max_size = self.MAX_DATA_SIZE_LMX
        CfgData = ffi.new("unsigned int a[]", self.max_size)
        self.logger.debug(f"\nret = XRFClk_GetConfigFromOneChip({ChipId}, {CfgData})")
        ret = rfclk_handle.XRFClk_GetConfigFromOneChip(ChipId, CfgData)
        self.logger.debug(f"The return value is {ret}")
        self.logger.debug(f"The Configuration data is the following:")
        print("[{}]".format(", ".join(hex(x) for x in CfgData)))
        return list(CfgData)

    def XRFClk_SetConfigOnAllChipsFromConfigId(
        self, ConfigId_LMK, ConfigId_1, ConfigId_2
    ):
        self.logger.debug(
            f"\nret = XRFClk_SetConfigOnAllChipsFromConfigId({ConfigId_LMK}, {ConfigId_1}, {ConfigId_2}"
        )
        ret = rfclk_handle.XRFClk_SetConfigOnAllChipsFromConfigId(
            ConfigId_LMK, ConfigId_1, ConfigId_2
        )
        self.logger.debug(f"The return value is: {ret}")
        return ret

    def XRFClk_WriteReg(self, ChipId, Data):
        self.logger.debug(f"\nret = XRFClk_WriteReg({ChipId}, {hex(Data)})")
        ret = rfclk_handle.XRFClk_WriteReg(ChipId, Data)
        self.logger.debug(f"The return value is: {ret}")
        return ret

    def XRFClk_ReadReg(self, ChipId):
        self.logger.debug(f"\nret = XRFClk_ReadReg({ChipId})")
        DataPtr = ffi.new("u32 *")
        ret = rfclk_handle.XRFClk_ReadReg(ChipId, DataPtr)
        Data = DataPtr[0]
        self.logger.debug(f"The return value is {ret} and Data is {Data}")
        return Data

    def __del__(self):
        self.logger.info("Inside RFCLK Destructor")



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Dragan Cvetic"
__copyright__ = "Copyright 2022, Xilinx"

import sys
sys.path.append('../../utils')
sys.path.append('../../raft_services')
import Pyro4
from console_server import CONSOLE
from utils import get_ip_and_port

IPADDR, PORT = get_ip_and_port()
if len(IPADDR) == 0:
    print("RAFT ERROR: Unable to Run Pyro Server.\n"
          "No network interface present.\n"
          "Please pass the ipaddress to __init__.py and retry.\n"
          "Usage:python3 __init__.py <ipaddress> <port>\n")
    sys.exit()
else:
    print("RAFT Pyro Server run successfully\n")

CONSOLE = Pyro4.expose(CONSOLE)

Pyro4.Daemon.serveSimple(
    {
        CONSOLE: "CONSOLE",
    },
    host=IPADDR,
    port=PORT,
    ns=False,
    verbose=True,
)



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Hugh Maguire"
__copyright__ = "Copyright 2021, Xilinx"

import sys
RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
import Pyro4
import logging
import base64
from cffi import FFI
from utils import ffi, xhelper_handle, open_c_library, cdata_to_py
from utils import extract_b64_encoded_string, cdata_string_to_py

sysmon_handle = open_c_library(RAFT_DIR+"xserver/xcffi/drv_header/pat/xsysmon_python.h", "/usr/lib/libXSysmon.so.1")

@Pyro4.expose
class SYSMON(object):

    def __init__(self):
        ret = xhelper_handle.XHelper_MetalInit(xhelper_handle.METAL_LOG_ERROR)
        if 0 != ret:
            logging.error("SYSMON: XHelper_MetalInit failed. ret = ", ret)
        logging.info("Inside Sysmon Constructor")
        pass

    def XSysmon_ReadValue(self, idStr):
        """
        API Read a INA226 register.

        :param addr: addr of INA226 device
        :param reg:  register to be read
        :return: ret_code, pointer unsigned short

        """
        logging.info("Inside XSysmon_ReadValue")
        idStr = idStr.encode('ascii')

        Val_ptr = ffi.new("float *")

        ret = sysmon_handle.XSysmon_ReadValue(idStr, Val_ptr)
        return ret, Val_ptr[0]



    def __del__(self):
        logging.info("Inside Sysmon Destructor")

#if __name__ == "__main__" :
#        Sysmon = SYSMON()
#        ret, val = Sysmon.XSysmon_ReadValue("in_temp0_ps_temp")



# Copyright (C) 2023-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2025, Advanced Micro Devices, Inc."

import os
import sys
import logging
import subprocess
from periphery import I2C

RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
sys.path.append(RAFT_DIR + 'xserver/raft_services/power_management/devices')
from pm_types import *

logging.basicConfig(level=logging.ERROR)

class Sysmon(object):
    def __init__(self, address, devicepath, devicetype):
        logging.info("Inside SysmonI2C Constructor")
        self._address = None
        self._bus = None
        self._min = 0
        self._min_min = 0
        self._max_max = 0
        self._temp = 0
        self._device_type = devicetype
        self._device_path = devicepath

        if self._device_type is Sysmon_Device_Type.I2C:
            self._address = int(address, 0)
            self._bus = I2C(self._device_path)
            try:
                msgs = [I2C.Message([0xc6, 0xd7, 0xe8, 0xf9, 0x03, 0x00, 0x08, 0x00])]
                self._bus.transfer(self._address, msgs)
            except IOError:
                logging.error("Sysmon I2C device init failed.")
                raise Exception("Sysmon I2C device init failed")

        elif self._device_type is Sysmon_Device_Type.PMBUS:
            self._address = int(address, 0)
            self._bus = I2C(self._device_path)
            try:
                msgs = [I2C.Message([0xc6, 0xd7, 0xe8, 0xf9, 0x03, 0x00, 0x08, 0x00])]
                self._bus.transfer(self._address, msgs)
            except IOError:
                logging.error("Sysmon Pmbus device init failed.")
                raise Exception("Sysmon Pmbus device init failed")

        elif self._device_type is Sysmon_Device_Type.SYSFS:
            for root, subdirs, files in os.walk("/sys/bus/iio/devices/"):
                for subdir in subdirs:
                    filepath = os.path.join(root, subdir, "name")
                    if os.path.isfile(filepath):
                        with open(filepath, 'r') as f:
                            file_content = f.read()
                            if 'xlnx,versal-sysmon' in file_content:
                                temp_path = os.path.join(root, subdir)
                                device_path = os.path.abspath(temp_path)
                                self._device_path = device_path
            if self._device_path is None:
                logging.error("Sysmon Sysfs device init failed.")
                raise Exception("Sysmon Sysfs device init failed.")
        else:
            logging.error("Unknown Sysmon device type.")
            raise Exception("Unknown Sysmon device type.")

    @staticmethod
    def twos_comp(val, bits):
        if (val & (1 << (bits - 1))) != 0:
            val = val - (1 << bits)
        return val

    @staticmethod
    def ConvertRawtoProcessed(first_byte, second_byte):
        val1 = (second_byte << 8) + first_byte
        if (val1 & 0x8000) == 0x8000:
            val1 = Sysmon.twos_comp(val1, 16)
        val2 = 128
        return float(val1)/float(val2)

    def ReadSysmonTemperatures(self):
        """
        Read Processing System Temperature Values

        :return: values of temp, min, max_max and min_min
        """
        if self._device_type == Sysmon_Device_Type.SYSFS:
            if self._device_path is not None:
                cmd  = 'cat ' + os.path.join(self._device_path, 'in_temp160_temp_input')
                ret, val = subprocess.getstatusoutput(cmd)
                if ret == 0:
                    self._temp = round(int(val)/1000, 3)

                cmd  = 'cat ' + os.path.join(self._device_path, 'in_temp161_min_input')
                ret, val = subprocess.getstatusoutput(cmd)
                if ret == 0:
                    self._min = round(int(val)/1000, 3)

                cmd  = 'cat ' + os.path.join(self._device_path, 'in_temp162_max_max_input')
                ret, val = subprocess.getstatusoutput(cmd)
                if ret == 0:
                    self._max_max = round(int(val)/1000, 3)

                cmd  = 'cat ' + os.path.join(self._device_path, 'in_temp163_min_min_input')
                ret, val = subprocess.getstatusoutput(cmd)
                if ret == 0:
                    self._min_min = round(int(val)/1000, 3)
            else:
                logging.error("Sysmon device path is None.")

        elif self._device_type == Sysmon_Device_Type.I2C:
            try:
                msgs = [I2C.Message([0xc6, 0xd7, 0xe8, 0xf9, 0x03, 0x00, 0x08, 0x00])]
                self._bus.transfer(self._address, msgs)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0x0c, 0x04, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._min = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0x0d, 0x04, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._temp = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0xe3, 0x07, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._min_min = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0xe4, 0x07, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._max_max = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

            except IOError:
                logging.error("Read Sysmon failed.")

        elif self._device_type == Sysmon_Device_Type.PMBUS:
            try:
                msgs = [I2C.Message([0xc6, 0xd7, 0xe8, 0xf9, 0x03, 0x00, 0x08, 0x00])]
                self._bus.transfer(self._address, msgs)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0x0c, 0x04, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._min = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0x0d, 0x04, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._temp = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0xe3, 0x07, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._min_min = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

                msgs = [I2C.Message([0x00, 0x00, 0x00, 0x00, 0xe4, 0x07, 0x04, 0x00]), I2C.Message([0x00, 0x00, 0x00, 0x00], read=True)]
                self._bus.transfer(self._address, msgs)
                logging.debug("{0}: 0x{1:02x}{2:02x}{2:02x}{3:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1], msgs[1].data[2], msgs[1].data[3]))

                self._max_max = round(self.ConvertRawtoProcessed(msgs[1].data[0], msgs[1].data[1]), 3)

            except IOError:
                logging.error("Read Sysmon failed.")
        else:
            ps_path  = os.path.join(self._device_path, self._address)
            ret, val = subprocess.getstatusoutput(["cat", ps_path])
            if ret == 0:
                self._temp = int(val)

        return self._temp, self._min, self._max_max, self._min_min

    def __del__(self):
        logging.info("Inside Sysmon Destructor")



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Hugh Maguire"
__copyright__ = "Copyright 2021, Xilinx"

import Pyro4
import logging
#import base64

logging.basicConfig(level=logging.ERROR)

class Sysmon_Client(object):
    Sysmon = None

    def __init__(self):
        logging.info("Inside Sysmon_Client Constructor")
        pass

    def SetIpAndPort(self, ipaddr, port):
        """
        API to inform Sysmon Client the IP address and port number of I2C Server.

        :param ipaddr: IP Address string
        :param port: Port number string
        :return: None

        """
        logging.debug("ipaddr = " + str(ipaddr))
        logging.debug("port = " + str(port))
        uri = "PYRO:SYSMON@" + str(ipaddr) + ":" + str(port)
        logging.debug("uri = " + uri)
        self.Sysmon = Pyro4.Proxy(uri)
        pass


    def XSysmon_ReadValue(self, idStr):
        """
        I2C driver read a register

        :param addr: address of INA226 device
        :param reg: register to be read
        :return: ret_code, value of register
        """
        ret, val = self.Sysmon.XSysmon_ReadValue(idStr)
        return ret, val

    def __del__(self):
        logging.info("Inside Sysmon Destructor")

sysmon = Sysmon_Client()



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Dragan Cvetic"
__copyright__ = "Copyright 2022, Xilinx"

import sys
import subprocess
RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
import logging
from utils import xhelper_handle
from utils import get_python_log_levels


class CONSOLE(object):
    device_id = 0
    logger = None

    def __init__(self):
        self.logger = self.GetLogger()
        ret = xhelper_handle.XHelper_MetalInit(xhelper_handle.METAL_LOG_ERROR)
        if 0 != ret:
            self.logger.error(f"CONSOLE: XHelper_MetalInit failed. ret = {ret}")
        self.logger.info("Inside CONSOLE Constructor")
        pass

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            logger.handler_set = True
            logger.disabled = False
        return logger

    # Log level
    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        return get_python_log_levels()

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"PythonLogLevel = {PythonLogLevel}")
        LogLevelsDict = get_python_log_levels()
        if PythonLogLevel == LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def RaftConsole(self, str_cmd):
        """
        API console command.

        :param : string as a "cat" command argument 
        :return: outStr output from cat command
        """
        self.logger.debug(f"execute: " + str_cmd)
        ret = subprocess.getstatusoutput(str_cmd)
        status = ret[0]
        str_cmd_ret = ret[1]
        self.logger.debug(f"return: status: {status}, command output: {str_cmd_ret}")
        return status, str_cmd_ret

    def __del__(self):
        self.logger.info("Inside CONSOLE Destructor")



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

import Pyro4
import logging
import base64
import json
import sys


class EQU_Client(object):
    EQU = None
    logger = None
    LogLevelsDict = {
        "DEBUG": 4,
        "INFO": 3,
        "WARNING": 2,
        "ERROR": 1,
        "CRITICAL": 0
    }

    def __init__(self):
        self.logger = self.GetLogger()
        self.logger.info("Inside RFDC Pyro Client Constructor")
        return

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            handler = logging.StreamHandler(sys.stdout)
            logger.addHandler(handler)
            logger.handler_set = True
            logger.disabled = False
        return logger

    @staticmethod
    def extract_b64_encoded_string(DictName):
        b64_data = DictName['data']
        b64_bytes_data = b64_data.encode('ascii')
        msg_bytes_data = base64.b64decode(b64_bytes_data)
        data = msg_bytes_data.decode('ascii')
        data = bytes(data, 'utf-8')
        return data

    def SetIpAndPort(self, ipaddr, port):
        """
        API to inform EQU Client the IP address and port number of EQU Server.

        :param ipaddr: IP Address string
        :param port: Port number string
        :return: None

        """
        uri = f"PYRO:EQU@{ipaddr}:{port}"
        self.logger.debug(f"SetIpAndPort({ipaddr}, {port})\n uri = {uri}")
        self.EQU = Pyro4.Proxy(uri)
        pass

    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        self.logger.debug("GetPythonLogLevels()")
        PythonLogLevel = self.EQU.GetPythonLogLevels()
        self.logger.debug(f"PythonLogLevel = {json.dumps(PythonLogLevel, indent=2)}")
        return PythonLogLevel

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"SetServerLogLevel({PythonLogLevel})")
        self.EQU.SetServerLogLevel(PythonLogLevel)
        return

    def SetClientLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        if PythonLogLevel == self.LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == self.LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == self.LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == self.LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def SetMetalLogLevel(self, MetalLogLevel):
        """
        Set the metal log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"SetMetalLogLevel({MetalLogLevel})")
        self.EQU.SetMetalLogLevel(MetalLogLevel)
        return

    def GetEquMacro(self):
        """
        Return Dictionary with all EQU macros in the equ header file

        :param : None
        :return: Dictionary with all EQU macros in the equ header file
        """
        self.logger.debug("GetEquMacro()")
        equ_macro = self.EQU.GetEquMacro()
        self.logger.debug(f"equ_macro = {json.dumps(equ_macro, indent=2)}")
        return equ_macro

    # Get enum API
    def GetEnum_XDfeEqu_StateId(self):
        """
        Return Dictionary equivalent of enum XDfeEqu_StateId

        :param : None
        :return: Dictionary equivalent of enum XDfeEqu_StateId
        """
        self.logger.debug("GetEnum_XDfeEqu_StateId()")
        XDfeEqu_StateId = self.EQU.GetEnum_XDfeEqu_StateId()
        self.logger.debug(f"XDfeEqu_StateId = {json.dumps(XDfeEqu_StateId, indent=2)}")
        return XDfeEqu_StateId

    def GetEnum_metal_log_level(self):
        """
        Return Dictionary equivalent of enum metal_log_level

        :param : None
        :return: Dictionary equivalent of enum metal_log_level
        """
        self.logger.debug("GetEnum_metal_log_level()")
        metal_log_level = self.EQU.GetEnum_metal_log_level()
        self.logger.debug(f"metal_log_level = {json.dumps(metal_log_level, indent=2)}")
        return metal_log_level

    # Get structure  API
    def GetStruct_XDfeEqu_Version(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_Version

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_Version
        """
        self.logger.debug("GetStruct_XDfeEqu_Version()")
        XDfeEqu_Version = self.EQU.GetStruct_XDfeEqu_Version()
        return XDfeEqu_Version

    def GetStruct_XDfeEqu_Trigger(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_Trigger

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_Trigger
        """
        self.logger.debug("GetStruct_XDfeEqu_Trigger()")
        XDfeEqu_Trigger = self.EQU.GetStruct_XDfeEqu_Trigger()
        return XDfeEqu_Trigger

    def GetStruct_XDfeEqu_TriggerCfg(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_TriggerCfg

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_TriggerCfg
        """
        self.logger.debug("GetStruct_XDfeEqu_TriggerCfg()")
        XDfeEqu_TriggerCfg = self.EQU.GetStruct_XDfeEqu_TriggerCfg()
        return XDfeEqu_TriggerCfg

    def GetStruct_XDfeEqu_ModelParameters(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_ModelParameters

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_ModelParameters
        """
        self.logger.debug("GetStruct_XDfeEqu_ModelParameters()")
        XDfeEqu_ModelParameters = self.EQU.GetStruct_XDfeEqu_ModelParameters()
        return XDfeEqu_ModelParameters

    def GetStruct_XDfeEqu_Cfg(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_Cfg

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_Cfg
        """
        self.logger.debug("GetStruct_XDfeEqu_Cfg()")
        XDfeEqu_Cfg = self.EQU.GetStruct_XDfeEqu_Cfg()
        return XDfeEqu_Cfg

    def GetStruct_XDfeEqu_Coefficients(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_Coefficients

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_Coefficients
        """
        self.logger.debug("GetStruct_XDfeEqu_Coefficients()")
        XDfeEqu_Coefficients = self.EQU.GetStruct_XDfeEqu_Coefficients()
        return XDfeEqu_Coefficients

    def GetStruct_XDfeEqu_EqConfig(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_EqConfig

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_EqConfig
        """
        self.logger.debug("GetStruct_XDfeEqu_EqConfig()")
        XDfeEqu_EqConfig = self.EQU.GetStruct_XDfeEqu_EqConfig()
        return XDfeEqu_EqConfig

    def GetStruct_XDfeEqu_Status(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_Status

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_Status
        """
        self.logger.debug("GetStruct_XDfeEqu_Status()")
        XDfeEqu_Status = self.EQU.GetStruct_XDfeEqu_Status()
        return XDfeEqu_Status

    def GetStruct_XDfeEqu_InterruptMask(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_InterruptMask

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_InterruptMask
        """
        self.logger.debug("GetStruct_XDfeEqu_InterruptMask()")
        XDfeEqu_InterruptMask = self.EQU.GetStruct_XDfeEqu_InterruptMask()
        return XDfeEqu_InterruptMask

    def GetStruct_XDfeEqu_Config(self):
        """
        Return Dictionary equivalent of structure XDfeEqu_Config

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu_Config
        """
        self.logger.debug("GetStruct_XDfeEqu_Config()")
        XDfeEqu_Config = self.EQU.GetStruct_XDfeEqu_Config()
        return XDfeEqu_Config

    def GetStruct_XDfeEqu(self):
        """
        Return Dictionary equivalent of structure XDfeEqu

        :param : None
        :return: Dictionary equivalent of structure XDfeEqu
        """
        self.logger.debug("GetStruct_XDfeEqu()")
        XDfeEqu = self.EQU.GetStruct_XDfeEqu()
        return XDfeEqu

    # System initialization API
    def XDfeEqu_InstanceInit(self, DeviceNodeName):
        """
        API initialises one instance of an Equalizer driver.
        Traverse "/sys/bus/platform/device" directory (in Linux), to find registered
        EQU device with the name DeviceNodeName. The first available slot in
        the instances array XDfeEqu_ChFilter[] will be taken as a DeviceNodeName
        object. On success it moves the state machine to a Ready state, while on
        failure stays in a Not Ready state.

        :param DeviceNodeName: device node name.
        :return: ret - 0 on success, 1 on failure
                 device_id - integer handle to initialized instance
                 DeviceNodeNameRet - device node name returned from driver
                                     which will be same as the passed value

        """
        self.logger.info(f"XDfeEqu_InstanceInit({DeviceNodeName})")
        DeviceNodeName = bytes(str(DeviceNodeName), 'utf-8')
        ret, device_id, DeviceNodeName = self.EQU.XDfeEqu_InstanceInit(DeviceNodeName)
        DeviceNodeName = self.extract_b64_encoded_string(DeviceNodeName)
        self.logger.info(f"ret = {ret}, device_id = {device_id}, DeviceNodeName = {DeviceNodeName}")
        return ret, device_id, DeviceNodeName

    def XDfeEqu_InstanceClose(self, device_id):
        """
        API closes the instance of an Equalizer driver.

        :param device_id: id of the opened device.
        :return: None
        """
        self.logger.info(f"XDfeEqu_InstanceClose({device_id})")
        self.EQU.XDfeEqu_InstanceClose(device_id)
        return

    # Register access API
    def XDfeEqu_WriteReg(self, device_id, addr_offset, data):
        """
        Writes value to register in an Equalizer instance.

        :param device_id: id of the opened device.
        :param AddrOffset: is address offset relative to instance base address.
        :param Data: is value to be written.
        :return: None
        """
        self.logger.debug(f"XDfeEqu_WriteReg({device_id}, {hex(addr_offset)}, {hex(data)})")
        self.EQU.XDfeEqu_WriteReg(device_id, addr_offset, data)
        return

    def XDfeEqu_ReadReg(self, device_id, addr_offset):
        """
        Reads a value from the register using an Equalizer instance.

        :param device_id: id of the opened device.
        :param AddrOffset: is address offset relative to instance base address.
        :return: Register value.
        """
        self.logger.debug(f"XDfeEqu_ReadReg({device_id}, {hex(addr_offset)})")
        regval = self.EQU.XDfeEqu_ReadReg(device_id, addr_offset)
        self.logger.debug(f"regval = {regval}")
        return regval

    # DFE Equalizer component initialization API
    def XDfeEqu_Reset(self, device_id):
        """
        Resets Equalizer and puts block into a reset state.

        :param device_id: id of the opened device.
        :return: None
        """
        self.logger.debug(f"XDfeEqu_Reset({device_id})")
        self.EQU.XDfeEqu_Reset(device_id)
        return

    def XDfeEqu_Configure(self, device_id, Cfg):
        """
        Reads configuration from device tree/xparameters.h and IP registers.
        Removes S/W reset and moves the state machine to a Configured state.

        :param device_id: id of the opened device.
        :param Cfg: device config container.
        :return: Cfg: device config container.
        """
        self.logger.debug(f"XDfeEqu_Configure({device_id}, {json.dumps(Cfg, indent=2)})")
        Cfg = self.EQU.XDfeEqu_Configure(device_id, Cfg)
        self.logger.debug(f"Cfg = {json.dumps(Cfg, indent=2)}")
        return Cfg

    def XDfeEqu_Initialize(self, device_id, EqConfig):
        """
        DFE Equalizer driver one time initialisation and moves the state machine to
        an Initialised state.

        :param device_id: id of the opened device.
        :param Config: configuration data container.
        :return: Config: configuration data container.
        """
        self.logger.debug(f"XDfeEqu_Initialize({device_id}, {json.dumps(EqConfig, indent=2)})")
        EqConfig = self.EQU.XDfeEqu_Initialize(device_id, EqConfig)
        self.logger.debug(f"Init = {json.dumps(EqConfig, indent=2)}")
        return EqConfig

    def XDfeEqu_Activate(self, device_id, EnableLowPower):
        """
        Activates channel Equalizer moves the state machine to an Activated state.

        :param device_id: id of the opened device.
        :param EnableLowPower: flag indicating low power.
        :return: None
        """
        self.logger.debug(f"XDfeEqu_Activate({device_id}, {EnableLowPower})")
        self.EQU.XDfeEqu_Activate(device_id, EnableLowPower)
        return

    def XDfeEqu_Deactivate(self, device_id):
        """
        Deactivates Equalizer and moves the state machine to Initialised state.

        :param device_id: id of the opened device.
        :return: None
        """
        self.logger.debug(f"XDfeEqu_Deactivate({device_id})")
        self.EQU.XDfeEqu_Deactivate(device_id)
        return

    def XDfeEqu_GetStateID(self, device_id):
        """
        Gets a state machine state id. The state id is returned as a string.
        The enum is mapped to a dictionary in python.

        :param device_id: id of the opened device.
        :return: StateId: State machine StateID as a string
        """
        self.logger.debug(f"XDfeEqu_GetStateID({device_id})")
        StateId = self.EQU.XDfeEqu_GetStateID(device_id)
        self.logger.debug(f"Return value StateId = {StateId}")
        return StateId

    # User APIs
    def XDfeEqu_Update(self, device_id, Config):
        """
        The software first sets bits 7:4 of the Next Control register (0x24) with
        the values in Config.Real_Datapath_Set, Config.Im_Datapath_Set.
        In real mode bits, 5:4 are set to the value held in
        Config.Real_Datapath_Set.
        Bits 7:6 are set to the value held in Config.Real_Datapath_Set.
        In complex mode bits, 5:4 are set to the value held in value
        Config.Real_Datapath_Set. Bits 7:6 are set to the value held in
        Config.Real_Datapath_Set plus 1.
        In matrix mode bits 5:4 are set to the value held in
        Config.Real_Datapath_Set. Bits 7:6 are set to the value held in
        Config.Im_Datapath_Set.
        The software sets bit 1 depending on Config.Flush.
        The software then sets the _Enable bit in the Next Control Trigger
        Source register (0x28) high.

        :param device_id: id of the opened device.
        :param Config: configuration container
        :return: Config: configuration container
        """
        self.logger.debug(f"XDfeEqu_Update({device_id}, {json.dumps(Config, indent=2)})")
        Config = self.EQU.XDfeEqu_Update(device_id, Config)
        self.logger.debug(f"Config = {json.dumps(Config, indent=2)}")
        return Config

    def XDfeEqu_GetTriggersCfg(self, device_id):
        """
        Returns current trigger configuration.

        :param device_id: id of the opened device.
        :return: TriggerCfg: is a trigger configuration container.
        """
        self.logger.debug(f"XDfeEqu_GetTriggersCfg({device_id})")
        TriggerCfg = self.EQU.XDfeEqu_GetTriggersCfg(device_id)
        self.logger.debug(f"TriggerCfg = {json.dumps(TriggerCfg, indent=2)}")
        return TriggerCfg

    def XDfeEqu_SetTriggersCfg(self, device_id, TriggerCfg):
        """
        Set trigger configuration.

        :param device_id: id of the opened device.
        :param TriggerCfg: is a triger configuration container.
        :return: TriggerCfg: is a triger configuration container.
        """
        self.logger.debug(f"XDfeEqu_SetTriggersCfg({device_id}, {json.dumps(TriggerCfg, indent=2)})")
        TriggerCfg = self.EQU.XDfeEqu_SetTriggersCfg(device_id, TriggerCfg)
        self.logger.debug(f"The return value TriggerCfg = {json.dumps(TriggerCfg, indent=2)}")
        return TriggerCfg

    def XDfeEqu_LoadCoefficients(self, device_id, ChannelField, Mode, Shift, EqCoeffs):
        """
        Sets Equalizer filter coefficients in Real, Complex or Matrix mode.

        :param device_id: id of the opened device.
        :param ChannelField: is a flag in which bits indicate the channel is enabled.
        :param Mode: is an equalizer mode.
        :param Shift: is a coefficient shift value.
        :param EqCoeffs: is equalizer coefficients container.
        :return: EqCoeffs: is equalizer coefficients container.
        """
        self.logger.debug(f"XDfeEqu_LoadCoefficients({device_id}, {ChannelField}, {Mode}, {Shift}, {json.dumps(EqCoeffs, indent=2)})")
        EqCoeffs = self.EQU.XDfeEqu_LoadCoefficients(
            device_id, ChannelField, Mode, Shift, EqCoeffs
        )
        self.logger.debug(f"EqCoeffs = {EqCoeffs}")
        return EqCoeffs

    def XDfeEqu_GetEventStatus(self, device_id):
        """
        Gets Equalizer event status

        :param device_id: id of the opened device.
        :return: Status: event status
        """
        self.logger.debug(f"XDfeEqu_GetEventStatus({device_id})")
        Status = self.EQU.XDfeEqu_GetEventStatus(device_id)
        self.logger.debug(f"Status = {json.dumps(Status, indent=2)}")
        return Status

    def XDfeEqu_ClearEventStatus(self, device_id, Status):
        """
        Clears Equalizer status. The channel status will be cleared for any IStatus
        or QStatus not equal 0.
        Note: The Status registers are only present for a given channel when it
              is present in the configured IP. The number of channels present is
              given by CONFIG.NUM_CHANNELS.        

        :param device_id: id of the opened device.
        :param Status: event status.
        :return: None
        """
        self.logger.debug(f"XDfeEqu_ClearEventStatus({device_id}, {json.dumps(Status, indent=2)})")
        self.EQU.XDfeEqu_ClearEventStatus(device_id, Status)
        return

    def XDfeEqu_SetInterruptMask(self, device_id, InterruptMask):
        """
        Enables an Equalizer status for channel ID.
        Note: The Status Mask registers are only present for a given channel when
              it is present in the configured IP. The number of channels present
              is given by CONFIG.NUM_CHANNELS.

        :param device_id: id of the opened device.
        :param InterruptMask: interrupt mask value
        :return: None
        """
        self.logger.debug(f"XDfeEqu_SetInterruptMask({device_id}, {json.dumps(InterruptMask, indent=2)})")
        self.EQU.XDfeEqu_SetInterruptMask(device_id, InterruptMask)
        return

    def XDfeEqu_GetInterruptMask(self, device_id):
        """
        Gets interrupt mask.

        :param device_id: id of the opened device.
        :return: InterruptMask: Equalizer interrupt mask container.
        """
        self.logger.debug(f"XDfeEqu_GetInterruptMask({device_id})")
        InterruptMask = self.EQU.XDfeEqu_GetInterruptMask(device_id)
        self.logger.debug(f"InterruptMask = {json.dumps(InterruptMask, indent=2)}")
        return InterruptMask

    def XDfeEqu_GetActiveSets(self, device_id):
        """
        Gets used coefficients settings.

        :param device_id: id of the opened device.
        :return: RealSet: Real value
                 ImagSet: Imaginary value
        """
        self.logger.debug(f"XDfeEqu_GetActiveSets({device_id})")
        RealSet, ImagSet = self.EQU.XDfeEqu_GetActiveSets(device_id)
        self.logger.debug(f"RealSet = {json.dumps(RealSet, indent=2)}, ImagSet = {json.dumps(ImagSet, indent=2)}")
        return RealSet, ImagSet

    def XDfeEqu_SetTUserDelay(self, device_id, Delay):
        """
        Sets the delay, which will be added to TUSER and TLAST (delay matched
        through the IP).

        :param device_id: id of the opened device.
        :param Delay: requested delay variable.
        :return: None
        """
        self.logger.debug(f"XDfeEqu_SetTUserDelay({device_id}, {Delay})")
        self.EQU.XDfeEqu_SetTUserDelay(device_id, Delay)
        return

    def XDfeEqu_GetTUserDelay(self, device_id):
        """
        Reads the delay, which will be added to TUSER and TLAST (delay matched
        through the IP).

        :param device_id: id of the opened device.
        :return: ret: Delay value
        """
        self.logger.debug(f"XDfeEqu_GetTUserDelay({device_id})")
        ret = self.EQU.XDfeEqu_GetTUserDelay(device_id)
        self.logger.debug(f"ret = {ret}")
        return ret

    def XDfeEqu_GetTDataDelay(self, device_id, Tap):
        """
        Returns CONFIG.DATA_LATENCY.VALUE + tap, where the tap is between 0
        and 23 in real mode and between 0 and 11 in complex/matrix mode.

        :param device_id: id of the opened device.
        :param Tap: is a tap variable.
        :return: ret: data latency value.
        """
        self.logger.debug(f"XDfeEqu_GetTDataDelay({device_id}, {Tap})")
        ret = self.EQU.XDfeEqu_GetTDataDelay(device_id, Tap)
        self.logger.debug(f"ret = {ret}")
        return ret

    def XDfeEqu_GetVersions(self, device_id):
        """
        This API is used to get the driver version.

        :param device_id: id of the opened device.
        :return: SwVersion: driver version numbers.
                 HwVersion: HW version numbers.
        """
        self.logger.debug(f"XDfeEqu_GetVersions({device_id})")
        SwVersion, HwVersion = self.EQU.XDfeEqu_GetVersions(device_id)
        self.logger.debug(f"SwVersion = {json.dumps(SwVersion, indent=2)}")
        self.logger.debug(f"HwVersion = {json.dumps(HwVersion, indent=2)}")
        return SwVersion, HwVersion

equ = EQU_Client()



# Copyright (C) 2021-2024 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Gerard Thomas Colman"
__copyright__ = "Copyright 2021, Xilinx"

import sys
def usage():
    # sys.argv[0] - usage python file name
    # sys.argv[1] - xclient/xcffi/xpyro
    # sys.argv[2] - host/board
    # sys.argv[3] - ip_address
    # sys.argv[4] - port
    print(f"Usage: The application work in three modes xclient, xcffi and xpyro\n"
          f"python3 {sys.argv[0]} xclient host/board ip_address port => Runs at host or board with xclient\n"
          f"sudo python3 {sys.argv[0]} xcffi => Runs at board with xcffi\n"
          f"python3 {sys.argv[0]} xpyro host/board ip_address port => Runs at host or board with xpyro\n"
          f"Example: python3 {sys.argv[0]} xclient host 169.254.10.2 9090")

# # 'xcffi' option will run only in the board
if (len(sys.argv) ==  2):
    if (sys.argv[1] != 'xcffi'):
        usage()
        sys.exit()

elif (len(sys.argv) !=  5):
    print("here")
    usage()
    sys.exit()

# The 'xclient' option will run both in host and board
if (sys.argv[1] == 'xclient'):
    # For 'xclient' option the path of the xclient is required
    # The relative path in the host is given for host case
    if (sys.argv[2] == 'host'):
        sys.path.append('../../../xclient/rfdc')
    # The location of xclient in the filesystem is given for the board case
    elif (sys.argv[2] == 'board'):
        sys.path.append('/usr/share/raft/xclient/rfdc')
    else:
        usage()
        sys.exit()
    import rfclk_client
    handle = rfclk_client.rfclk
    ip_address = sys.argv[3]
    port = sys.argv[4]

elif (sys.argv[1] == 'xcffi'):
    # For 'xcffi' option the path of the cffi python code is required
    sys.path.append('/usr/share/raft/xserver/xcffi/drv_api/rfdc')
    from rfclk_server import RFCLK
    handle = RFCLK()

# The 'xpyro' option will run both in host and board
elif (sys.argv[1] == 'xpyro'):
    ip_address = sys.argv[3]
    port = sys.argv[4]
    import Pyro4
    # Prepare the uri needed for pyro communication
    uri = f"PYRO:RFCLK@{ip_address}:{port}"
    handle = Pyro4.Proxy(uri)
else:
    usage()
    sys.exit()

if (sys.argv[1] == 'xclient'):
    #SetIpAndPort
    handle.SetIpAndPort(ip_address, port)
    #GetPythonLogLevels
    PythonLogLevels = handle.GetPythonLogLevels()
    #SetClientLogLevel
    handle.SetClientLogLevel(PythonLogLevels["DEBUG"])

#GetPythonLogLevels
PythonLogLevels = handle.GetPythonLogLevels()

#SetServerLogLevel
handle.SetServerLogLevel(PythonLogLevels["DEBUG"])

#SetMetalLogLevel
metal_log_level = handle.GetEnum_metal_log_level()
handle.SetMetalLogLevel(metal_log_level["METAL_LOG_EMERGENCY"])

#GetRfclkMacro
rfclk_macro = handle.GetRfclkMacro()
print(rfclk_macro["XST_SUCCESS"])
print(rfclk_macro["XST_FAILURE"])
print(rfclk_macro["RFCLK_LMX2594_1"])
print(rfclk_macro["RFCLK_LMX2594_2"])
print(rfclk_macro["RFCLK_LMK"])
print(rfclk_macro["RFCLK_CHIP_NUM"])
print(rfclk_macro["LMK_COUNT"])
print(rfclk_macro["LMK_FREQ_NUM"])
print(rfclk_macro["LMX_ADC_NUM"])
print(rfclk_macro["LMX_DAC_NUM"])
print(rfclk_macro["LMX2594_COUNT"])
print(rfclk_macro["FREQ_LIST_STR_SIZE"])

#XRFClk_Init
ret = handle.XRFClk_Init(486)
print(ret)

#XRFClk_ResetChip
ret = handle.XRFClk_ResetChip(0)
print(ret)

#XRFClk_SetConfigOnOneChipFromConfigId
ret = handle.XRFClk_SetConfigOnOneChipFromConfigId(0,0)
print(ret)

#XRFClk_GetConfigFromOneChip
ret = handle.XRFClk_GetConfigFromOneChip(0)
print(ret)

#XRFClk_SetConfigOnAllChipsFromConfigId
ret = handle.XRFClk_SetConfigOnAllChipsFromConfigId(0,0,0)
print(ret)

#XRFClk_WriteReg
ret = handle.XRFClk_WriteReg(0,0)
print(ret)

#XRFClk_ReadReg
DataVal = handle.XRFClk_ReadReg(0)
print(DataVal)



# Copyright (C) 2023-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2025, Advanced Micro Devices, Inc."

import os
import sys
import json
import logging
sys.path.append('../../utils')
sys.path.append('../../raft_services')
sys.path.append('../../raft_services/power_management')
import Pyro4
from periphery import I2C
from pm import PM
from utils import get_ip_and_port

RAFT_DIR = '/usr/share/raft/'

logging.basicConfig(level=logging.ERROR)

IPADDR, PORT = get_ip_and_port()
if len(IPADDR) == 0:
    print("CRITITCAL ERROR: Unable to Run Pyro Server.\n"
          "No eth0 network interface present.\n"
          "Please pass the ipaddress to __init__.py and retry.\n"
          "Usage:python3 __init__.py <ipaddress> <port>\n")
    sys.exit(1)
else:
    print("RAFT Pyro Server run successfully\n")

def exit_program():
    logging.error("CRITICAL ERROR: Unable to Run Raft-PM Server.")
    sys.exit(1)

def is_valid_json_file(file_path):
    if not os.path.isfile(file_path):
        logging.error("ERROR:root:Board Identification Json is not in Path : {file_path}")
        return False
    try:
        with open(file_path, 'r') as file:
            data = json.load(file)
            return True
    except json.JSONDecodeError:
        logging.error("ERROR:root:{file} Board Identification is not a JSON")
        return False

class board_eeprom:
    Name = ""
    I2C_Bus = ""
    I2C_Addr = ""

onboard = board_eeprom()

def parse_json_file(json_file):
    with open(json_file, 'r') as f:
        json_data = json.load(f)
        #print(json.dumps(json_data, indent=2))
    return json_data

def find_i2c_device_by_name(target_name):
    sys_i2c_path = "/sys/bus/i2c/devices/"

    for device in os.listdir(sys_i2c_path):
        name_path = os.path.join(sys_i2c_path, device, "name")

        if os.path.isfile(name_path):
            try:
                with open(name_path, "r") as f:
                    device_name = f.read().strip()

                if device_name == target_name:
                    bus_number, address = device.split("-")
                    bus_number = bus_number.replace("i2c", "")
                    dev_path = f"/dev/i2c-{bus_number}"
                    logging.debug(f"Found {target_name} at {dev_path}, address 0x{address}")
                    return dev_path, int(address, 16)
            except IOError:
                continue

    logging.debug(f"Device {target_name} not found")
    return None, None

def get_eeprom_data():
    result = False
    i2c = None
    onboard.Name = "Common"
    onboard.I2C_Bus = "/dev/i2c-1"
    onboard.I2C_Addr = 0x54
    data  = bytearray(256)
    try:
        i2c = I2C(onboard.I2C_Bus)
        msgs = [I2C.Message([0x0, 0x0]), I2C.Message(data, read=True)]
        i2c.transfer(onboard.I2C_Addr, msgs)
        eeprom_data = msgs[1].data
        result = True
        i2c.close()
    except:
        logging.debug(f"Onboard {onboard.Name} Eeprom read failed!")

    if result is False:
        onboard.Name = "Legacy"
        onboard.I2C_Bus = "/dev/i2c-11"
        onboard.I2C_Addr = 0x54
        data  = bytearray(256)
        try:
            i2c = I2C(onboard.I2C_Bus)
            msgs = [I2C.Message([0x0, 0x0]), I2C.Message(data, read=True)]
            i2c.transfer(onboard.I2C_Addr, msgs)
            eeprom_data = msgs[1].data
            result = True
            i2c.close()
        except:
            logging.debug(f"Onboard {onboard.Name} Eeprom read failed!")

    if result is False:
        device_path, device_address = find_i2c_device_by_name("24c128")
        if device_path is not None:
            onboard.Name = "Custom"
            onboard.I2C_Bus = device_path
            onboard.I2C_Addr = device_address
            data  = bytearray(256)
            try:
                i2c = I2C(onboard.I2C_Bus)
                msgs = [I2C.Message([0x0, 0x0]), I2C.Message(data, read=True)]
                i2c.transfer(onboard.I2C_Addr, msgs)
                eeprom_data = msgs[1].data
                result = True
                i2c.close()
            except:
                logging.debug(f"Onboard {onboard.Name} Eeprom read failed!")

    if result is False:
        logging.error("Board Eeprom Identification Failed.")
        exit_program()

    return eeprom_data

def get_product_name():
    eeprom_data = get_eeprom_data()
    offset = 0x15
    length = int.from_bytes(eeprom_data[offset:offset+1], "big") & 0x3f
    name = eeprom_data[offset+1:(offset+1 + length)].decode("utf-8").strip('\x00')
    return name

json_file = os.path.join(RAFT_DIR, 'xserver/raft_services/power_management/board', '.'.join((get_product_name(), 'json')))

if is_valid_json_file(json_file):
    json_data = parse_json_file(json_file)
    board_name = os.path.splitext(os.path.basename(json_file))
else:
    exit_program()

PM = Pyro4.expose(PM)

Pyro4.Daemon.serveSimple(
    {
        PM(json_data, board_name[0], onboard): "PM",
    },
    host=IPADDR,
    port=PORT,
    ns=False,
    verbose=True,
)



# Copyright (C) 2023-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2025, Advanced Micro Devices, Inc."

import os
import psutil
import subprocess

SYSMON_IIO_DIR = '/sys/bus/iio/devices/iio:device0/'

class Stats(object):

    @staticmethod
    def path2channel(channel_name):
        return SYSMON_IIO_DIR + channel_name
        #return os.path.join(SYSMON_IIO_DIR, '/'.join(channel_name))

    def getCpuUtilization(self):
        cpu_percent_per_core = psutil.cpu_percent(interval=1, percpu=True)
        cpu_utilization = {"CPU Utilization": {}}
        for i, cpu_percent in enumerate(cpu_percent_per_core):
            cpu_utilization["CPU Utilization"][f"CPU Core {i + 1}"] = cpu_percent
        return cpu_utilization

    def getCpuFrequencies(self):
        cpu_frequencies = {"CPU Frequencies": {}}
        for i, freq in enumerate(psutil.cpu_freq(percpu=True)):
            cpu_frequencies["CPU Frequencies"][f"CPU Core {i + 1}"] = {
                "Current Frequency": freq.current,
                "Max Frequency": freq.max,
                "Min Frequency": freq.min
            }
        return cpu_frequencies

    def getMemoryUsage(self):
        memory = psutil.virtual_memory()
        used_memory = memory.used
        free_memory = memory.free
        total_memory = memory.total

        with open('/proc/meminfo') as meminfo_file:
            meminfo_data = meminfo_file.read()

        cma_total_line = next(line for line in meminfo_data.split('\n') if 'CmaTotal' in line)
        cma_free_line = next(line for line in meminfo_data.split('\n') if 'CmaFree' in line)

        cma_total = int(cma_total_line.split()[1])
        cma_free = int(cma_free_line.split()[1])

        swap = psutil.swap_memory()
        swap_free = swap.free
        swap_total = swap.total
        swap_percent = swap.percent

        memory_usage = {
            "Memory Usage": {
                "Used Memory": used_memory / (1024 ** 2),
                "Free Memory": free_memory / (1024 ** 2),
                "Total Memory": total_memory / (1024 ** 2)
            },
            "CMA Mem Utilization": {
                "Cma Total" : cma_total,
                "Cma Free" : cma_free
            },
            "Swap Mem Utilization": {
                "Swap Free": swap_free / (1024 ** 2),
                "Swap Total": swap_total / (1024 ** 2),
                "Swap Usage": swap_percent
            }
        }
        return memory_usage

    def getVoltages(self):
        voltages = {"Voltage Info": {
            "PS Sysmon": {},
            "PL Sysmon": {},
            "AMS CTRL": {}
        }}
        voltages
        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage9_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage9_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC PS LPD": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage10_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage10_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC PS FPD": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage11_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage11_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"PS AUX": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage12_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage12_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"DDR IO VCC": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage13_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage13_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"PS IO BANK 503": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage14_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage14_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"PS IO BANK 500": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage15_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage15_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCCO_PSIO1": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage16_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage16_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCCO_PSIO2": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage17_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage17_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC_PS_GTR": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage18_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage18_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VTT_PS_GTR": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage19_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage19_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC_PSADC": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage21_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage21_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCINT": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage22_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage22_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCAUX": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage23_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage23_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"ADC Ref P+": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage24_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage24_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"ADC Ref N-": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage25_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage25_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCBRAM": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage26_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage26_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCC_PSINTLP": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage27_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage27_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCC_PSINTFP": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage28_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage28_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"PS AUX": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage29_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage29_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCAMS": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage0_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage0_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSPLL": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage1_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage1_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSBATT": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage2_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage2_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCCINT": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage3_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage3_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCCBRAM": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage4_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage4_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCCAUX": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage5_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage5_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSDDR_PLL": val})

        ret1, raw = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage6_raw'))
        ret2, scale = subprocess.getstatusoutput('cat ' + self.path2channel('in_voltage6_scale'))
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSINTFP_DDR": val})
        return voltages

    def GetTemperatures(self):
        temps = {"Temperature Info": {}}
        ret1, pl_temp = subprocess.getstatusoutput('cat ' + self.path2channel("in_temp20_input"))
        ret2, lpd_temp = subprocess.getstatusoutput('cat ' + self.path2channel('in_temp7_input'))
        ret3, fpd_temp = subprocess.getstatusoutput('cat ' + self.path2channel('in_temp8_input'))
        if (ret1 | ret2 | ret3) == 0:
            temps["Temperature Info"]["FPD Temp"] = float(int(fpd_temp)/1000)
            temps["Temperature Info"]["LPD Temp"] = float(int(lpd_temp)/1000)
            temps["Temperature Info"]["PL Temp"] = float(int(pl_temp)/1000)
        return temps

    def getLastStatus(self):
        status = {}
        status.update(self.getCpuUtilization())
        status.update(self.getCpuFrequencies())
        status.update(self.getMemoryUsage())
        status.update(self.GetTemperatures())
        status.update(self.getVoltages())
        return status


# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

import Pyro4
import serpent
import logging

def convert_to_bytearray(data):
    b = bytearray()
    for i in range(len(data)):
        c = data[i].to_bytes(4, byteorder='little', signed=False)
        b.extend(c)
    return b

def convert_to_list(data):
    k = []
    for i in range(int(len(data)/4)):
        k.append(int.from_bytes(data[(i*4):(((i+1)*4))], byteorder='little'))
    return k


class AXI_MEMMAP_Client(object):
    AXI_MEMMAP = None

    def __init__(self):
        logging.info("Inside AXI_MEMMAP Pyro Client Constructor")
        pass

    def SetIpAndPort(self, ipaddr, port):
        """
        API to inform AXI_MEMMAP Client the IP address and port number of AXI_MEMMAP Server.

        :param ipaddr: IP Address string
        :param port: Port number string
        :return: None

        """
        uri = f"PYRO:AXI_MEMMAP@{ipaddr}:{port}"
        logging.debug(f"SetIpAndPort({ipaddr}, {port})\n uri = {uri}")
        self.AXI_MEMMAP = Pyro4.Proxy(uri)
        pass

    def axi_read_words(self, address, num_words, network_order):
        """
        Read num_words from address.

        :param address: address to read from
        :param num_words: number of words to read
        :param network_order: Apply network byte ordering if network_order!=0
        :return: ret: whether success or failure
                 buf: the read data

        """
        logging.debug("axi_read_words({address}, {num_words}, {network_order})")
        ret, buf = self.AXI_MEMMAP.axi_read_words(address, num_words, network_order)
        logging.debug(f"ret = {ret}")
        buf = serpent.tobytes(buf)
        buf = convert_to_list(buf)
        return ret, buf

    def axi_write_words(self, address, num_words, buf, network_order):
        """
        Write num_words to address.

        :param address: address to write
        :param num_words: number of words to write
        :param buffer: data to write
        :param network_order: Apply network byte ordering if network_order!=0
        :return: ret: whether success or failure

        """
        buf = convert_to_bytearray(buf)
        logging.debug("axi_write_words({address}, {num_words}, {network_order})")
        ret = self.AXI_MEMMAP.axi_write_words(address, num_words, buf, network_order)
        logging.debug(f"ret = {ret}")
        return ret

axi_memmap = AXI_MEMMAP_Client()



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2021, Xilinx"

from rftool import Rftool
import matplotlib.pyplot as plt
import numpy as np
from scipy import fft

rftool_handle = Rftool()
# SetBoardName
board_name = "zcu208"
rftool_handle.SetBoardName(board_name)
# SetIpAndPort
rftool_handle.SetIpAndPort("169.254.10.2", "9090")
# SetMetalLogLevel
metal_log_level = rftool_handle.GetEnum_metal_log_level()
rftool_handle.SetMetalLogLevel(metal_log_level["METAL_LOG_DEBUG"])
# GetPythonLogLevels
PythonLogLevels = rftool_handle.GetPythonLogLevels()
# SetClientLogLevel
rftool_handle.SetClientLogLevel(PythonLogLevels["ERROR"])

ret, inst_id = rftool_handle.Initialize()
# setup clocking
DistributionSettings = rftool_handle.GetStruct_XRFdc_Distribution_Settings()
if board_name == "zcu208":
    DistributionSettings["SourceTileId"] = 0
else:
    DistributionSettings["SourceTileId"] = 2
DistributionSettings["SourceType"] = 1
DistributionSettings["EdgeTypes"] = [0,1]
DistributionSettings["EdgeTileIds"] = [0,3]
DistributionSettings["DistRefClkFreq"] = 245.76
DistributionSettings["DistributedClock"] = 1
DistributionSettings["SampleRates"] = [[2211.86, 2211.86, 2211.86, 2211.86], [2211.86, 2211.86, 2211.86, 2211.86]]
ret = rftool_handle.XRFdc_SetClkDistribution(inst_id, DistributionSettings)

MixerSettingsADC = {
    "Freq": 0.0,
    "PhaseOffset": 0.0,
    "EventSource": 2,
    "CoarseMixFreq": 0x10,
    "MixerMode": 4,
    "FineMixerScale": 0,
    "MixerType": 1,
}
tile_id = 0
dac_block_id = 0
if board_name == "zcu208":
    if dac_block_id == 2:
        adc_block_id = 1
    else:
        adc_block_id = 0
else:
    adc_block_id = dac_block_id

N = 2048  # number of 16bit samples
f = 502.560
FS = 2211.86
FS_DAC = 2211.86
T = 1/FS_DAC
t = np.arange(0, (N * T), T)
sig = 0x1fff * np.cos(2 * np.pi * f * t)

ret, MixerSettingsADC = rftool_handle.XRFdc_SetMixerSettings(inst_id, 0, tile_id, adc_block_id, MixerSettingsADC)
print("rftool_handle.XRFdc_SetMixerSettings ret = ", ret)

ret = rftool_handle.XRFdc_UpdateEvent(inst_id, 0, tile_id, adc_block_id, 1)
print("rftool_handle.XRFdc_UpdateEvent ret = ", ret)

ret, df = rftool_handle.XRFdc_GetDecimationFactor(inst_id, tile_id, adc_block_id)
print(f"rftool_handle.XRFdc_GetDecimationFactor ret = {ret} Decimation factor = {df}")

ret = rftool_handle.SetMMCM(inst_id, 1, 0)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 1, 1)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 1, 2)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 1, 3)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 0, 0)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 0, 1)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 0, 2)
print("rftool_handle.SetMMCM ret = ", ret)

ret = rftool_handle.SetMMCM(inst_id, 0, 3)
print("rftool_handle.SetMMCM ret = ", ret)


# Ensure Channels Enabled
#First Tiles must be disabled (see pg269)
rftool_handle.lmem_wr32(0xb000000c, 0x0)
rftool_handle.lmem_wr32(0xb040000c, 0x0)
#Channels can now be enabled
rftool_handle.lmem_wr32(0xb0000008, 0xffff)
rftool_handle.lmem_wr32(0xb0400008, 0xffff)
#Enable all Tiles
rftool_handle.lmem_wr32(0xb000000c, 0xf)
rftool_handle.lmem_wr32(0xb040000c, 0xf)

rftool_handle.WriteDataToMemory(tile_id, dac_block_id, sig)
rftool_handle.SetLocalMemSample(1, tile_id, dac_block_id, N)
rftool_handle.LocalMemTrigger(1)

rftool_handle.SetLocalMemSample(0, tile_id, adc_block_id, N)
rftool_handle.LocalMemTrigger(0)
SigOut = rftool_handle.ReadDataFromMemory(tile_id, adc_block_id, N, 0)

rftool_handle.SetLocalMemSample(0, tile_id, adc_block_id, N)
rftool_handle.LocalMemTrigger(0)
SigOut = rftool_handle.ReadDataFromMemory(tile_id, adc_block_id, N, 0)

dft = np.abs(fft.fft(SigOut[0]))
fstp = FS/N
fx = np.arange(0, FS, fstp)
plt.plot(fx, dft)
plt.show()



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Anish Kadamathikuttiyil Karthikeyan Pillai"
__copyright__ = "Copyright 2022, Xilinx"

import sys
def usage():
    # sys.argv[0] - usage python file name
    # sys.argv[1] - xclient/xcffi/xpyro
    # sys.argv[2] - host/board
    # sys.argv[3] - ip_address
    # sys.argv[4] - port
    print(f"Usage: The application work in three modes xclient, xcffi and xpyro\n"
          f"python3 {sys.argv[0]} xclient host/board ip_address port => Runs at host or board with xclient\n"
          f"sudo python3 {sys.argv[0]} xcffi => Runs at board with xcffi\n"
          f"python3 {sys.argv[0]} xpyro host/board ip_address port => Runs at host or board with xpyro\n"
          f"Example: python3 {sys.argv[0]} xclient host 169.254.10.2 9090")

# 'xcffi' option will run only in the board
if (len(sys.argv) ==  2):
    if (sys.argv[1] != 'xcffi'):
        usage()
        sys.exit()

elif (len(sys.argv) !=  5):
    usage()
    sys.exit()

# The 'xclient' option will run both in host and board
if (sys.argv[1] == 'xclient'):
    # For 'xclient' option the path of the xclient is required
    # The relative path in the host is given for host case
    if (sys.argv[2] == 'host'):
        sys.path.append('../../../xclient/dfe')
    # The location of xclient in the filesystem is given for the board case
    elif (sys.argv[2] == 'board'):
        sys.path.append('/usr/share/raft/xclient/dfe')
    else:
        usage()
        sys.exit()
    import ofdm_client
    handle = ofdm_client.ofdm
    ip_address = sys.argv[3]
    port = sys.argv[4]

elif (sys.argv[1] == 'xcffi'):
    # For 'xcffi' option the path of the cffi python code is required
    sys.path.append('/usr/share/raft/xserver/xcffi/drv_api/dfe')
    from ofdm_server import OFDM
    handle = OFDM()

# The 'xpyro' option will run both in host and board
elif (sys.argv[1] == 'xpyro'):
    ip_address = sys.argv[3]
    port = sys.argv[4]
    import Pyro4
    # Prepare the uri needed for pyro communication
    uri = f"PYRO:OFDM@{ip_address}:{port}"
    handle = Pyro4.Proxy(uri)
else:
    usage()
    sys.exit()

if (sys.argv[1] == 'xclient'):
    #SetIpAndPort
    handle.SetIpAndPort(ip_address, port)
    #GetPythonLogLevels
    PythonLogLevels = handle.GetPythonLogLevels()
    #SetClientLogLevel
    handle.SetClientLogLevel(PythonLogLevels["DEBUG"])

#GetPythonLogLevels
PythonLogLevels = handle.GetPythonLogLevels()

#SetServerLogLevel
handle.SetServerLogLevel(PythonLogLevels["DEBUG"])

#SetMetalLogLevel
metal_log_level = handle.GetEnum_metal_log_level()
handle.SetMetalLogLevel(metal_log_level["METAL_LOG_DEBUG"])

#GetOfdmMacro
ofdm_macro = handle.GetOfdmMacro()
print(ofdm_macro["XDFEOFDM_MAX_NUM_INSTANCES"])
print(ofdm_macro["XST_SUCCESS"])
print(ofdm_macro["XST_FAILURE"])
print(ofdm_macro["XDFEOFDM_NODE_NAME_MAX_LENGTH"])
print(ofdm_macro["XDFEOFDM_CC_NUM"])
print(ofdm_macro["XDFEOFDM_FT_NUM"])
print(ofdm_macro["XDFEOFDM_CC_SEQ_LENGTH_MAX"])
print(ofdm_macro["XDFEOFDM_FT_SEQ_LENGTH_MAX"])
print(ofdm_macro["XDFEOFDM_PHASE_COMPENSATION_MAX"])

#GetEnum_XDfeOfdm_StateId
XDfeOfdm_StateId = handle.GetEnum_XDfeOfdm_StateId()

#GetEnum_metal_log_level
metal_log_level = handle.GetEnum_metal_log_level()

#GetStruct_XDfeOfdm_InternalCarrierCfg
XDfeOfdm_InternalCarrierCfg = handle.GetStruct_XDfeOfdm_InternalCarrierCfg()

#XDfeOfdm_InstanceInit
#XDfeOfdm *XDfeOfdm_InstanceInit(const char *DeviceNodeName);
#The bytes conversion is already done inside xpyro client.
DeviceNodeName = "a7e40000.xdfe_ofdm"

ret, device_id, DeviceNodeName = handle.XDfeOfdm_InstanceInit(DeviceNodeName)

#XDfeOfdm_WriteReg
#void XDfeOfdm_WriteReg(const XDfeOfdm *InstancePtr, u32 AddrOffset, u32 Data);
handle.XDfeOfdm_WriteReg(device_id, 0x40, 0)

#XDfeOfdm_ReadReg
#u32 XDfeOfdm_ReadReg(const XDfeOfdm *InstancePtr, u32 AddrOffset);
ret = handle.XDfeOfdm_ReadReg(device_id, 0)

#XDfeOfdm_Reset
#void XDfeOfdm_Reset(XDfeOfdm *InstancePtr);
handle.XDfeOfdm_Reset(device_id)

#XDfeOfdm_Configure
#void XDfeOfdm_Configure(XDfeOfdm *InstancePtr, XDfeOfdm_Cfg *Cfg);
Cfg_in = handle.GetStruct_XDfeOfdm_Cfg()
Cfg_out = handle.XDfeOfdm_Configure(device_id, Cfg_in)

#XDfeOfdm_Initialize
#void XDfeOfdm_Initialize(XDfeOfdm *InstancePtr, const XDfeOfdm_Init *Init);
Init_in = handle.GetStruct_XDfeOfdm_Init()
Init_in['CCSequenceLength']=6
Init_out = handle.XDfeOfdm_Initialize(device_id, Init_in)

#XDfeOfdm_SetTriggersCfg
#void XDfeOfdm_SetTriggersCfg(const XDfeCcf *InstancePtr, XDfeOfdm_TriggerCfg *TriggerCfg);
TriggerCfg_in = handle.GetStruct_XDfeOfdm_TriggerCfg()
TriggerCfg_in['Activate']['TriggerEnable'] = 0
TriggerCfg_in['Activate']['Mode'] = 0
TriggerCfg_in['Activate']['TuserEdgeLevel'] = 0
TriggerCfg_in['Activate']['StateOutput'] = 0
TriggerCfg_in['Activate']['TUSERBit'] = 0
TriggerCfg_in['CCUpdate']['TriggerEnable'] = 0
TriggerCfg_in['CCUpdate']['Mode'] = 0
TriggerCfg_in['CCUpdate']['TuserEdgeLevel'] = 0
TriggerCfg_in['CCUpdate']['StateOutput'] = 0
TriggerCfg_in['CCUpdate']['TUSERBit'] = 0
TriggerCfg_out = handle.XDfeOfdm_SetTriggersCfg(device_id, TriggerCfg_in)

#XDfeOfdm_Activate
#void XDfeOfdm_Activate(XDfeCcf *InstancePtr, bool EnableLowPower);
handle.XDfeOfdm_Activate(device_id, 0)

#XDfeOfdm_GetStateID
#XDfeOfdm_StateId XDfeOfdm_GetStateID(XDfeCcf *InstancePtr);
StateId = handle.XDfeOfdm_GetStateID(device_id)
print(StateId)

#XDfeOfdm_GetCurrentCCCfg
#void XDfeOfdm_GetCurrentCCCfg(const XDfeCcf *InstancePtr, XDfeOfdm_CCCfg *CCCfg);
CurrCCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
CurrentCCCfg = handle.XDfeOfdm_GetCurrentCCCfg(device_id, CurrCCCfg)

# XDfeOfdm_GetEmptyCCCfg
# void XDfeOfdm_GetEmptyCCCfg(const XDfeCcf *InstancePtr, XDfeOfdm_CCCfg *CCCfg);
CCCfg = handle.XDfeOfdm_GetEmptyCCCfg(device_id) #TODO

# XDfeOfdm_GetCarrierCfg
# void XDfeOfdm_GetCarrierCfg(const XDfeCcf *InstancePtr, XDfeOfdm_CCCfg *CCCfg,
#			   s32 CCID, u32 *CCSeqBitmap,
#			   XDfeOfdm_CarrierCfg *CarrierCfg);
CCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
CCSeqBitmap, CarrierCfg = handle.XDfeOfdm_GetCarrierCfg(device_id, CCCfg, 0) #TODO

# XDfeOfdm_AddCCtoCCCfg
# u32 XDfeOfdm_AddCCtoCCCfg(XDfeCcf *InstancePtr, XDfeOfdm_CCCfg *CCCfg, s32 CCID,
#			 u32 CCSeqBitmap, const XDfeOfdm_CarrierCfg *CarrierCfg);
CCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
CCCfg = handle.XDfeOfdm_GetCurrentCCCfg(device_id, CCCfg)
CCID = 0x1
CCSeqBitmap = 0x2
CarrierCfg = handle.GetStruct_XDfeOfdm_CarrierCfg()
Status = handle.GetStruct_XDfeOfdm_Status()
CarrierCfg['FftSize'] = 1024
Status['CCUpdate'] = 1
handle.XDfeOfdm_ClearEventStatus(int(device_id), Status)
FTSeq = handle.GetStruct_XDfeOfdm_FTSequence()
ret, CCCfg, FTSeq = handle.XDfeOfdm_AddCCtoCCCfg(int(device_id), CCCfg, CCID, CCSeqBitmap, CarrierCfg, FTSeq) #TODO

# XDfeOfdm_RemoveCCfromCCCfg
# void XDfeOfdm_RemoveCCfromCCCfg(XDfeCcf *InstancePtr, XDfeOfdm_CCCfg *CCCfg,
# 			       s32 CCID);
CCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
ret, CCCfg, FTSeq = handle.XDfeOfdm_RemoveCCfromCCCfg(device_id, CCCfg, 0, FTSeq)#TODO

# XDfeOfdm_UpdateCCinCCCfg
# void XDfeOfdm_UpdateCCinCCCfg(const XDfeCcf *InstancePtr, XDfeOfdm_CCCfg *CCCfg,
# 			     s32 CCID, const XDfeOfdm_CarrierCfg *CarrierCfg);
CCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
CarrierCfg = handle.GetStruct_XDfeOfdm_CarrierCfg()
#ret, CCCfg, FTSeq = handle.XDfeOfdm_UpdateCCinCCCfg(device_id, CCCfg, 0, CarrierCfg, FTSeq) #TODO

# XDfeOfdm_SetNextCCCfg
# void XDfeOfdm_SetNextCCCfg(const XDfeOfdm *InstancePtr,
#				  const XDfeOfdm_CCCfg *NextCCCfg);
CCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
CCCfg = handle.XDfeOfdm_SetNextCCCfg(device_id, CCCfg)
print(f"------------------------------->XDfeOfdm_SetNextCCCfg")
print(CCCfg)

# XDfeOfdm_EnableCCUpdateTrigger
# u32 XDfeOfdm_EnableCCUpdateTrigger(const XDfeOfdm *InstancePtr);
ret = handle.XDfeOfdm_EnableCCUpdateTrigger(device_id)
print(f"------------------------------->XDfeOfdm_EnableCCUpdateTrigger")
print(ret)

# XDfeOfdm_SetNextCCCfgAndTrigger
# u32 XDfeOfdm_SetNextCCCfgAndTrigger(const XDfeCcf *InstancePtr,
#				   XDfeOfdm_CCCfg *CCCfg);
CCCfg = handle.GetStruct_XDfeOfdm_CCCfg()
ret, CCCfg = handle.XDfeOfdm_SetNextCCCfgAndTrigger(device_id, CCCfg)#TODO

#XDfeOfdm_AddCC
#u32 XDfeOfdm_AddCC(const XDfeCcf *InstancePtr, u32 CCID,
#		  const XDfeOfdm_CarrierCfg *CarrierCfg);
Status = handle.GetStruct_XDfeOfdm_Status()
Status['CCUpdate'] = 1
handle.XDfeOfdm_ClearEventStatus(int(device_id), Status)
CarrierCfg_in = handle.GetStruct_XDfeOfdm_CarrierCfg()
CCSeqBitmap = 0xb
FTSeq = handle.GetStruct_XDfeOfdm_FTSequence()
FTSeq['Length'] = 2
FTSeq['CCID'] = [1,2,3,4, 0,0,0,0, 0,0,0,0, 0,0,0,0]
CCID = 0x5
CCSeqBitmap = 0x17
CarrierCfg_in = handle.GetStruct_XDfeOfdm_CarrierCfg()
CarrierCfg_in['Numerology'] = 1
CarrierCfg_in['NumSubcarriers'] = 0x5
CarrierCfg_in['ScaleFactor'] = 0x55
CarrierCfg_in['OutputDelay'] = 0x3455
CarrierCfg_in['FftSize'] = 1024
ret = handle.XDfeOfdm_AddCC(device_id, 0, CCSeqBitmap, CarrierCfg_in, FTSeq)

#XDfeOfdm_RemoveCC
#u32 XDfeOfdm_RemoveCC(const XDfeCcf *InstancePtr, u32 CCID);
ret = handle.XDfeOfdm_RemoveCC(device_id, 0, FTSeq)#TODO

#XDfeOfdm_UpdateCC
#u32 XDfeOfdm_UpdateCC(const XDfeCcf *InstancePtr, u32 CCID,
#                      XDfeOfdm_CarrierCfg *CarrierCfg);
CarrierCfg_in = handle.GetStruct_XDfeOfdm_CarrierCfg()
#ret, CarrierCfg_out = handle.XDfeOfdm_UpdateCC(device_id, 0, CarrierCfg_in, FTSeq)#TODO

#XDfeOfdm_GetTriggersCfg
#void XDfeOfdm_GetTriggersCfg(const XDfeCcf *InstancePtr,
#			                XDfeOfdm_TriggerCfg *TriggerCfg);
TriggerCfg_out = handle.XDfeOfdm_GetTriggersCfg(device_id)

# XDfeOfdm_SetTuserOutFrameLocation
#void XDfeOfdm_SetTuserOutFrameLocation(const XDfeOfdm *InstancePtr,
#                       u32 TuserOutFrameLocation);
TuserOutFrameLocation = 0x34
handle.XDfeOfdm_SetTuserOutFrameLocation(device_id, TuserOutFrameLocation)

#XDfeOfdm_GetTuserOutFrameLocation
#u32 XDfeOfdm_GetTuserOutFrameLocation(const XDfeOfdm *InstancePtr);
TuserOutFrameLocation = handle.XDfeOfdm_GetTuserOutFrameLocation(device_id)
assert TuserOutFrameLocation == 0x34

# XDfeOfdm_SetTuserCCUpdateTrigger
#void XDfeOfdm_SetTuserCCUpdateTrigger(const XDfeOfdm *InstancePtr,
#				       u32 TuserCCUpdateTrigger);
TuserCCUpdateTrigger = 0x34
handle.XDfeOfdm_SetTuserCCUpdateTrigger(device_id, TuserCCUpdateTrigger)

#XDfeOfdm_GetTuserCCUpdateTrigger
#u32 XDfeOfdm_GetTuserCCUpdateTrigger(const XDfeOfdm *InstancePtr);
TuserCCUpdateTrigger = handle.XDfeOfdm_GetTuserCCUpdateTrigger(device_id)
assert TuserCCUpdateTrigger == 0x34

#XDfeOfdm_GetEventStatus
#void XDfeOfdm_GetEventStatus(const XDfeCcf *InstancePtr, XDfeOfdm_Status *Status);
Status = handle.XDfeOfdm_GetEventStatus(device_id)

#XDfeOfdm_ClearEventStatus
#void XDfeOfdm_ClearEventStatus(const XDfeCcf *InstancePtr);
Status_in = handle.GetStruct_XDfeOfdm_Status()
handle.XDfeOfdm_ClearEventStatus(device_id, Status_in)

#XDfeOfdm_SetInterruptMask
#void XDfeOfdm_SetInterruptMask(const XDfeCcf *InstancePtr,
#	            		       const XDfeOfdm_InterruptMask *Mask);
Mask_in = handle.GetStruct_XDfeOfdm_InterruptMask()
handle.XDfeOfdm_SetInterruptMask(device_id, Mask_in)

#XDfeOfdm_GetInterruptMask
#void XDfeOfdm_GetInterruptMask(const XDfeCcf *InstancePtr,
#	            		       const XDfeOfdm_InterruptMask *Mask);
Mask_out = handle.XDfeOfdm_GetInterruptMask(device_id)

#XDfeOfdm_Deactivate
#void XDfeOfdm_Deactivate(XDfeCcf *InstancePtr);
handle.XDfeOfdm_Deactivate(device_id)

#void XDfeOfdm_SetTUserDelay(const XDfeOfdm *InstancePtr, u32 Delay);
TUserDelay = 0x123
handle.XDfeOfdm_SetTUserDelay(device_id, TUserDelay)

#u32 XDfeOfdm_GetTUserDelay(const XDfeOfdm *InstancePtr);
delay = handle.XDfeOfdm_GetTUserDelay(device_id)

#u32 XDfeOfdm_GetDataLatency(const XDfeOfdm *InstancePtr);
delay = handle.XDfeOfdm_GetDataLatency(device_id)

#XDfeOfdm_GetVersions
#void XDfeOfdm_GetVersions(const XDfeCcf *InstancePtr, XDfeOfdm_Version *SwVersion,
#			              XDfeOfdm_Version *HwVersion);
SwVersion_out, HwVersion_out = handle.XDfeOfdm_GetVersions(device_id)

#XDfeOfdm_InstanceClose
#void XDfeOfdm_InstanceClose(XDfeOfdm *InstancePtr);
handle.XDfeOfdm_InstanceClose(device_id)




# Copyright (C) 2023-2025 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2025, Advanced Micro Devices, Inc."

import logging
from periphery import I2C

PM_LOWLEVEL_DEBUG = False

def pm_print(printstr):
    if(PM_LOWLEVEL_DEBUG):
        print(printstr)

logging.basicConfig(level=logging.ERROR)

class INA226(object):
    CONFIGURATION = 0x00
    SHUNT_VOLTAGE = 0x01
    BUS_VOLTAGE =   0x02
    POWER =         0x03
    CURRENT =       0x04
    CALIBRATION =   0x05
    MASK_ENABLE =   0x06
    ALERT_LIMIT =   0x07
    MANUFACTURER =  0XFE
    DIE_ID =        0XFF
    # default values
    MANUFACTURER_ID_VAL = 0x5449
    DIE_ID_VAL =    0x2260

    def __init__(self, address, devicepath):
        logging.info("Inside INA226 Constructor")
        self.die_id = 0
        self.manu_id = 0
        self.addr = int(address, 0)
        self.i2c = I2C(devicepath)
        self._currentLSB = 0
        self._maxCurrent = 0
        self._shunt = 0
        self._phase_multiplier = 0
    
    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k in ['addr', 'i2c'] and v
        }
        return str(str_info)

    def initSensor(self, mc, sr, pm):
        self.die_id = self.getDieID()
        self.manu_id = self.getManufacturerID()
        if self.die_id == INA226.DIE_ID_VAL:
            if self.SetCalibration(mc, sr, pm):
                return True
            else:
                logging.error("INA226 Calibration Failed.")
                return False
        else:
            logging.error("INA226 init failed.")
            return False

    def getShuntVoltage(self):
        voltage = self._readRegister(INA226.SHUNT_VOLTAGE)
        voltage *= 2.5e-6
        return voltage

    def getBusVoltage(self):
        voltage = self._readRegister(INA226.BUS_VOLTAGE)
        voltage *= 1.25e-3
        return voltage

    def getPower(self):
        power = self._readRegister(INA226.POWER)
        power *= (self._currentLSB * 25)
        power *= self._phase_multiplier
        return power

    def getCurrent(self):
        value = self._readRegister(INA226.CURRENT)
        if value > 0x7fff:
            value -= 0x10000
            value = abs(value)

        current = self._currentLSB * value
        current *= self._phase_multiplier
        return current

    def SetCalibration(self, max_current, shunt_resistor, phase_multiplier):
        current_LSB = max_current / (32678 * 1000)
        calibration = (0.00512 * 1000000) / (current_LSB * shunt_resistor)

        if calibration > 32767:
            calibration = 32767

        self._writeRegister(INA226.CALIBRATION, int(calibration))
        calibration = self._readRegister(INA226.CALIBRATION)

        if calibration != 0:
            current_LSB = (0.00512 * 1000000) / (calibration * shunt_resistor)
        else:
            return False

        self._currentLSB = current_LSB
        self._maxCurrent = current_LSB * 32768
        self._shunt = shunt_resistor
        self._phase_multiplier = phase_multiplier
        return True

    def getManufacturerID(self):
        return self._readRegister(INA226.MANUFACTURER)

    def getDieID(self):
        return self._readRegister(INA226.DIE_ID)

    def readRegisterValues(self):
        registers = []
        for reg_adr in range(8):
            registers.append(self._readRegister(reg_adr))
        registers.append(self.manu_id)
        registers.append(self.die_id)
        for index, reg_val in enumerate(registers):
            pm_print("register[0x{0:02x}] = 0x{1:04x}".format(index, reg_val))
        return registers

    def writeRegisterValues(self, reg_val):
        if not reg_val[0] == None:
            pm_print("0x{0:04x}".format(reg_val[0]))
            self._writeRegister(INA226.CONFIGURATION, reg_val[0])
        if not reg_val[1] == None:
            pm_print("0x{0:04x}".format(reg_val[1]))
            self._writeRegister(INA226.CALIBRATION, reg_val[1])
        if not reg_val[2] == None:
            pm_print("0x{0:04x}".format(reg_val[2]))
            self._writeRegister(INA226.MASK_ENABLE, reg_val[2])
        if not reg_val[3] == None:
            pm_print("0x{0:04x}".format(reg_val[3]))
            self._writeRegister(INA226.ALERT_LIMIT, reg_val[3])

    def _readRegister(self, register):
        val = 0
        try:
            msgs = [I2C.Message([register]), I2C.Message([0x00, 0x00], read=True)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            pm_print("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            val = (msgs[1].data[0] << 8) + msgs[1].data[1]
        except IOError:
                logging.error("INA226 _readRegister failed.")
        return val

    def _writeRegister(self, register, value):
        data = None
        try:
            data = bytearray(value.to_bytes(2, "big"))
            data.insert(0, register)
            msgs = [I2C.Message(data)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[0].data[1], msgs[0].data[2]))
            pm_print("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[0].data[1], msgs[0].data[2]))
        except IOError:
            logging.error("INA226 _writeRegister failed.")

    def __del__(self):
        logging.info("Inside INA226 Destructor")

class INA7XX(object):
    CONFIG      = 0x00
    ADC_CONFIG  = 0x01
    VBUS        = 0x05 # readonly
    DIETEMP     = 0x06 # readonly
    CURRENT     = 0x07 # readonly
    POWER       = 0x08 # readonly
    ENERGY      = 0x09 # readonly
    CHARGE      = 0x0A # readonly
    DIAG_ALERT  = 0x0B
    COL         = 0x0C
    CUL         = 0x0D
    BOVL        = 0x0E
    BUVL        = 0x0F
    TEMP_LIMIT  = 0x10
    PWR_LIMIT   = 0x11
    MANUFACTURER_ID = 0x3E # readonly
    # register count
    REGISTERS   = 16
    MAX_REGISTERS = 16
    # default values
    CONFIG_DEFAULT      = 0x10
    ADC_CONFIG_DEFAULT  = 0xFB68 # averages=1
    VBUS_DEFAULT        = 0x00
    DIETEMP_DEFAULT     = 0x00
    CURRENT_DEFAULT     = 0x00
    POWER_DEFAULT       = 0x00
    ENERGY_DEFAULT      = 0x00
    CHARGE_DEFAULT      = 0x00
    DIAG_ALERT_DEFAULT  = 0x01
    COL_DEFAULT         = 0x7FFF
    CUL_DEFAULT         = 0x8000
    BOVL_DEFAULT        = 0x7FFF
    BUVL_DEFAULT        = 0x00
    TEMP_LIMIT_DEFAULT  = 0x7FFF
    PWR_LIMIT_DEFAULT   = 0xFFFF
    MANUFACTURER_ID_DEFAULT = 0x5449
    
    current_lsb = 0
    voltage_lsb = 0
    temperature_lsb = 0
    power_lsb = 0
    energy_lsb = 0
    charge_lsb = 0

    def __init__(self, address, devicepath):
        logging.info("Inside INA7XX Constructor")
        self.addr = int(address, 0)
        self.i2c = I2C(devicepath)

    def initSensor(self, mc, sr, pm):
        _ = mc
        _ = sr
        _ = pm
        if INA7XX.MANUFACTURER_ID_DEFAULT == self.getManufacturerID():
            return True
        else:
            return False

    def getBusVoltage(self):
        voltage = self._readRegister(INA7XX.VBUS)
        voltage *= self.voltage_lsb
        return voltage

    def getPower(self):
        power = self._readRegister24bit(INA7XX.POWER)
        power *= self.power_lsb
        return power

    def getCurrent(self):
        current = self._readRegister(INA7XX.CURRENT)
        if current & 0x8000:
            current -= 0x10000
        current *= self.current_lsb
        return current

    def getManufacturerID(self):
        return self._readRegister(INA7XX.MANUFACTURER_ID)

    def readRegisterValues(self):
        registers = []
        for reg_adr in range(18):
            if reg_adr in [2, 3, 4, 9, 10]:
                continue
            else:
                registers.append(self._readRegister(reg_adr))
        registers.append(self._readRegister(INA7XX.MANUFACTURER_ID))
        for index, reg_val in enumerate(registers):
            pm_print("register[0x{0:02x}] = 0x{1:04x}".format(index, reg_val))
        return registers

    def writeRegisterValues(self, reg_val):
        if not reg_val[0] == None:
            pm_print("0x{0:04x}".format(reg_val[0]))
            self._writeRegister(INA7XX.CONFIG, reg_val[0])
        if not reg_val[1] == None:
            pm_print("0x{0:04x}".format(reg_val[1]))
            self._writeRegister(INA7XX.ADC_CONFIG, reg_val[1])
        if not reg_val[2] == None:
            pm_print("0x{0:04x}".format(reg_val[2]))
            self._writeRegister(INA7XX.DIAG_ALERT, reg_val[2])
        if not reg_val[3] == None:
            pm_print("0x{0:04x}".format(reg_val[3]))
            self._writeRegister(INA7XX.TEMP_LIMIT, reg_val[3])
        if not reg_val[4] == None:
            pm_print("0x{0:04x}".format(reg_val[4]))
            self._writeRegister(INA7XX.PWR_LIMIT, reg_val[4])

    def _readRegister40bit(self, register):
        val = 0
        try:
            msgs = [I2C.Message([register]), I2C.Message([0x00, 0x00, 0x00, 0x00, 0x00], read=True)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            pm_print("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            val = (msgs[1].data[0] << 32) (msgs[1].data[1] << 24) (msgs[1].data[2] << 16) + (msgs[1].data[3] << 8) + msgs[1].data[4]
        except IOError:
                logging.error("INA7XX _readRegister failed.")
        return val

    def _readRegister24bit(self, register):
        val = 0
        try:
            msgs = [I2C.Message([register]), I2C.Message([0x00, 0x00, 0x00], read=True)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            pm_print("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            val = (msgs[1].data[0] << 16) + (msgs[1].data[1] << 8) + msgs[1].data[2]
        except IOError:
                logging.error("INA7XX _readRegister failed.")
        return val

    def _readRegister(self, register):
        val = 0
        try:
            msgs = [I2C.Message([register]), I2C.Message([0x00, 0x00], read=True)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            pm_print("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[1].data[0], msgs[1].data[1]))
            val = (msgs[1].data[0] << 8) + msgs[1].data[1]
        except IOError:
                logging.error("INA7XX _readRegister failed.")
        return val

    def _writeRegister(self, register, value):
        data = None
        try:
            data = bytearray(value.to_bytes(2, "big"))
            data.insert(0, register)
            msgs = [I2C.Message(data)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[0].data[1], msgs[0].data[2]))
            pm_print("{0}@0x{1:02x}: 0x{2:02x}{3:02x}".format(self.i2c.devpath, self.addr, msgs[0].data[1], msgs[0].data[2]))
        except IOError:
            logging.error("INA7XX _writeRegister failed.")

    def __del__(self):
        logging.info("Inside INA7XX Destructor")

class INA700(INA7XX):
    def __init__(self, address, devicepath):
        self.current_lsb = 480e-6
        self.voltage_lsb = 3.125e-3
        self.temperature_lsb = 125e-3
        self.power_lsb = 96e-6
        self.energy_lsb = 1.536e-3
        self.charge_lsb = 30e-6
        super().__init__(address, devicepath)


class INA745x(INA7XX):
    def __init__(self, address, devicepath):
        self.current_lsb = 1.2e-3
        self.voltage_lsb = 3.125e-3
        self.temperature_lsb = 125e-3
        self.power_lsb = 240e-6
        self.energy_lsb = 3.840e-3
        self.charge_lsb = 75e-6
        super().__init__(address, devicepath)



# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Dragan Cvetic"
__copyright__ = "Copyright 2022, Xilinx"

import Pyro4
import logging
import base64
import json
import sys


class CONSOLE_Client(object):
    CONSOLE = None
    logger = None
    LogLevelsDict = {
        "DEBUG": 4,
        "INFO": 3,
        "WARNING": 2,
        "ERROR": 1,
        "CRITICAL": 0
    }

    def __init__(self):
        self.logger = self.GetLogger()
        self.logger.info("Inside CONSOLE Pyro Client Constructor")
        return

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            handler = logging.StreamHandler(sys.stdout)
            logger.addHandler(handler)
            logger.handler_set = True
            logger.disabled = False
        return logger

    def SetIpAndPort(self, ipaddr, port):
        """
        API to inform CONSOLE Client the IP address and port number of CONSOLE Server.

        :param ipaddr: IP Address string
        :param port: Port number string
        :return: None

        """
        uri = f"PYRO:CONSOLE@{ipaddr}:{port}"
        self.logger.debug(f"SetIpAndPort({ipaddr}, {port})\n uri = {uri}")
        self.CONSOLE = Pyro4.Proxy(uri)
        pass

    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        self.logger.debug("GetPythonLogLevels()")
        PythonLogLevel = self.CONSOLE.GetPythonLogLevels()
        self.logger.debug(f"PythonLogLevel = {json.dumps(PythonLogLevel, indent=2)}")
        return PythonLogLevel

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"SetServerLogLevel({PythonLogLevel})")
        self.CONSOLE.SetServerLogLevel(PythonLogLevel)
        return

    def SetClientLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """

        if PythonLogLevel == self.LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == self.LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == self.LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == self.LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def SetMetalLogLevel(self, MetalLogLevel):
        """
        Set the metal log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"SetMetalLogLevel({MetalLogLevel})")
        self.CONSOLE.SetMetalLogLevel(MetalLogLevel)
        return

    def RaftConsole(self, strCmd):
        """
        API console command.

        :param : string as a "cat" command argument 
        :return: outStr output from cat command
        """
        self.logger.debug(f"execute: " + strCmd)
        status, strCmdRet = self.CONSOLE.RaftConsole(strCmd)
        self.logger.debug(f"return: ststus: {status}, command output: {strCmdRet}")
        return status, strCmdRet

console = CONSOLE_Client()
